@@ -14,6 +14,10 @@
 
 #include "bindcfg.h"
 
+#if LIBDNS_VERSION_MAJOR < 1600
+#define cfg_parse_buffer cfg_parse_buffer4
+#endif
+
 cfg_type_t *cfg_type_update_policy;
 cfg_type_t *cfg_type_allow_query;
 cfg_type_t *cfg_type_allow_transfer;
@@ -108,7 +112,7 @@ cfg_parse_strbuf(cfg_parser_t *parser, const char *string, cfg_type_t **type,
 	isc_buffer_init(&buffer, (char *)string, string_len);
 	isc_buffer_add(&buffer, string_len);
 
-	result = cfg_parse_buffer(parser, &buffer, *type, &ret);
+	result = cfg_parse_buffer(parser, &buffer, NULL, 0, *type, 0, &ret);
 
 	if (result == ISC_R_SUCCESS)
 		*objp = ret;
