@@ -12,6 +12,10 @@
 #include "util.h"
 #include "zone_register.h"
 
+#if LIBDNS_VERSION_MAJOR < 1600
+#define dns_name_copynf(src, dst) dns_name_copy((src), (dst), NULL)
+#endif
+
 /**
  * These zones should not leak onto the Internet.
  * The list matches BIND commit 8f20f6c9d7ce5a0f0af6ee4c5361832d97b1c5d4
@@ -226,15 +230,13 @@ empty_zone_search_stop(empty_zone_search_t *iter) {
  * @returns @see empty_zone_search_next
  */
 isc_result_t
-empty_zone_search_init(empty_zone_search_t *iter, dns_name_t *qname,
+empty_zone_search_init(empty_zone_search_t *iter, const dns_name_t *qname,
                        dns_zt_t *ztable) {
-	isc_result_t result;
-
 	REQUIRE(iter != NULL);
 	REQUIRE(dns_name_isabsolute(qname));
 
 	INIT_BUFFERED_NAME(iter->qname);
-	CHECK(dns_name_copy(qname, &iter->qname, NULL));
+	dns_name_copynf(qname, &iter->qname);
 
 	INIT_BUFFERED_NAME(iter->ezname);
 	iter->nextidx = 0;
@@ -243,9 +245,6 @@ empty_zone_search_init(empty_zone_search_t *iter, dns_name_t *qname,
 	dns_zt_attach(ztable, &iter->zonetable);
 
 	return empty_zone_search_next(iter);
-
-cleanup:
-	return result;
 }
 
 /**
@@ -298,7 +297,7 @@ cleanup:
  *    it failed and user configured policy != only.
  */
 isc_result_t
-empty_zone_handle_conflicts(dns_name_t *name, dns_zt_t *zonetable,
+empty_zone_handle_conflicts(const dns_name_t *name, dns_zt_t *zonetable,
 			    bool warn_only)
 {
 	isc_result_t result;
