@@ -30,6 +30,7 @@
 #include <dns/result.h>
 #include <dns/soa.h>
 #include <dns/types.h>
+#include <dns/rpz.h>
 
 #include <string.h> /* For memcpy */
 
@@ -51,6 +52,17 @@
 #define VALID_LDAPDB(ldapdb) \
 	((ldapdb) != NULL && (ldapdb)->common.impmagic == LDAPDB_MAGIC)
 
+#if LIBDNS_VERSION_MAJOR < 1600
+typedef dns_name_t       node_name_t;
+#else
+typedef const dns_name_t node_name_t;
+#endif
+
+isc_result_t
+ldapdb_associate(isc_mem_t *mctx, node_name_t *name, dns_dbtype_t type,
+		 dns_rdataclass_t rdclass, unsigned int argc, char *argv[],
+		 void *driverarg, dns_db_t **dbp) ATTR_NONNULL(1,2,7,8);
+
 struct ldapdb {
 	dns_db_t			common;
 	isc_refcount_t			refs;
@@ -118,7 +130,11 @@ attach(dns_db_t *source, dns_db_t **targetp)
 
 	REQUIRE(VALID_LDAPDB(ldapdb));
 
+#if LIBDNS_VERSION_MAJOR < 1600
 	isc_refcount_increment(&ldapdb->refs, NULL);
+#else
+	isc_refcount_increment(&ldapdb->refs);
+#endif
 	*targetp = source;
 }
 
@@ -126,6 +142,8 @@ attach(dns_db_t *source, dns_db_t **targetp)
 static void ATTR_NONNULLS
 free_ldapdb(ldapdb_t *ldapdb)
 {
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
 #ifdef RBTDB_DEBUG
 	isc_result_t result;
 	dns_dbversion_t *version = NULL;
@@ -153,8 +171,8 @@ cleanup:
 #endif
 	dns_db_detach(&ldapdb->rbtdb);
 	dns_name_free(&ldapdb->common.origin, ldapdb->common.mctx);
-	RUNTIME_CHECK(isc_mutex_destroy(&ldapdb->newversion_lock)
-		      == ISC_R_SUCCESS);
+	/* isc_mutex_destroy is failing fatal now */
+	isc_mutex_destroy(&ldapdb->newversion_lock);
 	isc_mem_putanddetach(&ldapdb->common.mctx, ldapdb, sizeof(*ldapdb));
 }
 
@@ -162,17 +180,24 @@ cleanup:
 static void
 detach(dns_db_t **dbp)
 {
-	ldapdb_t *ldapdb = (ldapdb_t *)(*dbp);
+	ldapdb_t *ldapdb;
 	unsigned int refs;
 
-	REQUIRE(VALID_LDAPDB(ldapdb));
-
-	isc_refcount_decrement(&ldapdb->refs, &refs);
-
-	if (refs == 0)
-		free_ldapdb(ldapdb);
+	REQUIRE(dbp != NULL && VALID_LDAPDB((ldapdb_t *)(*dbp)));
 
+	ldapdb = (ldapdb_t *)(*dbp);
 	*dbp = NULL;
+
+#if LIBDNS_VERSION_MAJOR < 1600
+	isc_refcount_decrement(&ldapdb->refs, &refs);
+#else
+	/* isc_refcount_decrement only has one argument now */
+	refs = isc_refcount_decrement(&ldapdb->refs);
+#endif
+
+	if (refs == 1) {
+		free_ldapdb(ldapdb);
+	}
 }
 
 
@@ -322,7 +347,7 @@ closeversion(dns_db_t *db, dns_dbversion_t **versionp, bool commit)
 }
 
 static isc_result_t
-findnode(dns_db_t *db, dns_name_t *name, bool create,
+findnode(dns_db_t *db, node_name_t *name, bool create,
 	 dns_dbnode_t **nodep)
 {
 	ldapdb_t *ldapdb = (ldapdb_t *) db;
@@ -333,7 +358,7 @@ findnode(dns_db_t *db, dns_name_t *name, bool create,
 }
 
 static isc_result_t
-find(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
+find(dns_db_t *db, node_name_t *name, dns_dbversion_t *version,
      dns_rdatatype_t type, unsigned int options, isc_stdtime_t now,
      dns_dbnode_t **nodep, dns_name_t *foundname, dns_rdataset_t *rdataset,
      dns_rdataset_t *sigrdataset)
@@ -342,21 +367,29 @@ find(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
 
 	REQUIRE(VALID_LDAPDB(ldapdb));
 
-	return dns_db_find(ldapdb->rbtdb, name, version, type, options, now,
-			   nodep, foundname, rdataset, sigrdataset);
+	return dns_db_find(ldapdb->rbtdb, name, version, type,
+			   options, now, nodep, foundname, rdataset,
+			   sigrdataset);
 }
 
 static isc_result_t
-findzonecut(dns_db_t *db, dns_name_t *name, unsigned int options,
+findzonecut(dns_db_t *db, node_name_t *name, unsigned int options,
 	    isc_stdtime_t now, dns_dbnode_t **nodep, dns_name_t *foundname,
+#if LIBDNS_VERSION_MAJOR >= 1600
+	    dns_name_t *dcname,
+#endif
 	    dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset)
 {
 	ldapdb_t *ldapdb = (ldapdb_t *) db;
 
 	REQUIRE(VALID_LDAPDB(ldapdb));
 
-	return dns_db_findzonecut(ldapdb->rbtdb, name, options, now, nodep,
-				  foundname, rdataset, sigrdataset);
+	return dns_db_findzonecut(ldapdb->rbtdb, name, options,
+				  now, nodep, foundname,
+#if LIBDNS_VERSION_MAJOR >= 1600
+				  dcname,
+#endif
+				  rdataset, sigrdataset);
 }
 
 static void
@@ -473,9 +506,7 @@ node_isempty(dns_db_t *db, dns_dbnode_t *node, dns_dbversion_t *version,
 	char buff[DNS_NAME_FORMATSIZE];
 	isc_result_t result;
 
-	dns_fixedname_init(&fname);
-
-	CHECK(ldapdb_name_fromnode(node, dns_fixedname_name(&fname)));
+	CHECK(ldapdb_name_fromnode(node, dns_fixedname_initname(&fname)));
 
 	result = dns_db_allrdatasets(db, node, version, now, &rds_iter);
 	if (result == ISC_R_NOTFOUND) {
@@ -683,7 +714,7 @@ getnsec3parameters(dns_db_t *db, dns_dbversion_t *version,
 }
 
 static isc_result_t
-findnsec3node(dns_db_t *db, dns_name_t *name, bool create,
+findnsec3node(dns_db_t *db, node_name_t *name, bool create,
 	      dns_dbnode_t **nodep)
 {
 	ldapdb_t *ldapdb = (ldapdb_t *) db;
@@ -744,8 +775,9 @@ getrrsetstats(dns_db_t *db) {
 
 }
 
+#if LIBDNS_VERSION_MAJOR < 1600
 void
-rpz_attach(dns_db_t *db, dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num)
+rpz_attach(dns_db_t *db, dns_rpz_zones_t *rpzs, uint8_t rpz_num)
 {
 	ldapdb_t *ldapdb = (ldapdb_t *) db;
 
@@ -753,7 +785,25 @@ rpz_attach(dns_db_t *db, dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num)
 
 	dns_db_rpz_attach(ldapdb->rbtdb, rpzs, rpz_num);
 }
+#else
+void
+rpz_attach(dns_db_t *db, void *void_rpzs, uint8_t rpz_num)
+{
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+	dns_rpz_zones_t *rpzs = (dns_rpz_zones_t *) void_rpzs;
+	isc_result_t result;
 
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	rpzs->zones[rpz_num]->db_registered = true;
+	result = dns_db_updatenotify_register(ldapdb->rbtdb,
+					      dns_rpz_dbupdate_callback,
+					      rpzs->zones[rpz_num]);
+	REQUIRE(result == ISC_R_SUCCESS);
+}
+#endif
+
+/*
 isc_result_t
 rpz_ready(dns_db_t *db)
 {
@@ -763,9 +813,10 @@ rpz_ready(dns_db_t *db)
 
 	return dns_db_rpz_ready(ldapdb->rbtdb);
 }
+*/
 
 static isc_result_t
-findnodeext(dns_db_t *db, dns_name_t *name,
+findnodeext(dns_db_t *db, node_name_t *name,
 		   bool create, dns_clientinfomethods_t *methods,
 		   dns_clientinfo_t *clientinfo, dns_dbnode_t **nodep)
 {
@@ -778,7 +829,7 @@ findnodeext(dns_db_t *db, dns_name_t *name,
 }
 
 static isc_result_t
-findext(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
+findext(dns_db_t *db, node_name_t *name, dns_dbversion_t *version,
 	       dns_rdatatype_t type, unsigned int options, isc_stdtime_t now,
 	       dns_dbnode_t **nodep, dns_name_t *foundname,
 	       dns_clientinfomethods_t *methods, dns_clientinfo_t *clientinfo,
@@ -823,6 +874,39 @@ nodefullname(dns_db_t *db, dns_dbnode_t *node, dns_name_t *name)
 	return dns_db_nodefullname(ldapdb->rbtdb, node, name);
 }
 
+#ifdef HAVE_DNS_SERVESTALE
+static isc_result_t
+setservestalettl(dns_db_t *db, dns_ttl_t ttl) {
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	return dns_db_setservestalettl(ldapdb->rbtdb, ttl);
+}
+
+static isc_result_t
+getservestalettl(dns_db_t *db, dns_ttl_t *ttl) {
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	return dns_db_getservestalettl(ldapdb->rbtdb, ttl);
+}
+#endif
+
+#if LIBDNS_VERSION_MAJOR >= 1606
+/* Used for cache size adjustments, called by dns_cache_setcachesize.
+ * Just proxy to rbtdb implementation. */
+static isc_result_t
+adjusthashsize(dns_db_t *db, size_t size) {
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	return dns_db_adjusthashsize(ldapdb->rbtdb, size);
+}
+#endif
+
 static dns_dbmethods_t ldapdb_methods = {
 	attach,
 	detach,
@@ -862,13 +946,23 @@ static dns_dbmethods_t ldapdb_methods = {
 	isdnssec,
 	getrrsetstats,
 	rpz_attach,
-	rpz_ready,
+	NULL, /* rpz_ready */
 	findnodeext,
 	findext,
 	setcachestats,
 	hashsize,
 	nodefullname,
 	NULL, // getsize method not implemented (related BZ1353563)
+#ifdef HAVE_DNS_SERVESTALE
+	setservestalettl,
+	getservestalettl,
+#endif
+#if LIBDNS_VERSION_MAJOR >= 1600
+	NULL, /* setgluecachestats */
+#endif
+#if LIBDNS_VERSION_MAJOR >= 1606
+	adjusthashsize, /* adjusthashsize */
+#endif
 };
 
 isc_result_t ATTR_NONNULLS
@@ -918,7 +1012,7 @@ dns_ns_buildrdata(dns_name_t *origin, dns_name_t *ns_name,
  * @param[in] argv [0] is database instance name
  */
 isc_result_t
-ldapdb_associate(isc_mem_t *mctx, dns_name_t *name, dns_dbtype_t type,
+ldapdb_associate(isc_mem_t *mctx, node_name_t *name, dns_dbtype_t type,
 		 dns_rdataclass_t rdclass, unsigned int argc, char *argv[],
 		 void *driverarg, dns_db_t **dbp) {
 
@@ -958,14 +1052,14 @@ ldapdb_create(isc_mem_t *mctx, dns_name_t *name, dns_dbtype_t type,
 	REQUIRE(driverarg != NULL);
 	REQUIRE(dbp != NULL && *dbp == NULL);
 
-	CHECKED_MEM_GET_PTR(mctx, ldapdb);
+	ldapdb = isc_mem_get(mctx, sizeof(*(ldapdb)));
 	ZERO_PTR(ldapdb);
 
 	isc_mem_attach(mctx, &ldapdb->common.mctx);
-	CHECK(isc_mutex_init(&ldapdb->newversion_lock));
+	/* isc_mutex_init and isc_condition_init failures are now fatal */
+	isc_mutex_init(&ldapdb->newversion_lock);
 	lock_ready = true;
 	dns_name_init(&ldapdb->common.origin, NULL);
-	isc_ondestroy_init(&ldapdb->common.ondest);
 
 	ldapdb->common.magic = DNS_DB_MAGIC;
 	ldapdb->common.impmagic = LDAPDB_MAGIC;
@@ -976,7 +1070,7 @@ ldapdb_create(isc_mem_t *mctx, dns_name_t *name, dns_dbtype_t type,
 
 	CHECK(dns_name_dupwithoffsets(name, mctx, &ldapdb->common.origin));
 
-	CHECK(isc_refcount_init(&ldapdb->refs, 1));
+	isc_refcount_init(&ldapdb->refs, 1);
 	ldapdb->ldap_inst = driverarg;
 
 	CHECK(dns_db_create(mctx, "rbt", name, dns_dbtype_zone,
@@ -988,9 +1082,10 @@ ldapdb_create(isc_mem_t *mctx, dns_name_t *name, dns_dbtype_t type,
 
 cleanup:
 	if (ldapdb != NULL) {
-		if (lock_ready == true)
-			RUNTIME_CHECK(isc_mutex_destroy(&ldapdb->newversion_lock)
-				      == ISC_R_SUCCESS);
+		if (lock_ready == true) {
+			/* isc_mutex_destroy errors are now fatal */
+			isc_mutex_destroy(&ldapdb->newversion_lock);
+		}
 		if (dns_name_dynamic(&ldapdb->common.origin))
 			dns_name_free(&ldapdb->common.origin, mctx);
 
@@ -1064,11 +1159,10 @@ dyndb_init(isc_mem_t *mctx, const char *name, const char *parameters,
 		isc_lib_register();
 		isc_log_setcontext(dctx->lctx);
 		dns_log_setcontext(dctx->lctx);
+		isc_hash_set_initializer(dctx->hashinit);
 		log_debug(5, "registering library from dynamic ldap driver, %p != %p.", dctx->refvar, &isc_bind9);
 	}
 
-	isc_hash_set_initializer(dctx->hashinit);
-
 	log_debug(2, "registering dynamic ldap driver for %s.", name);
 
 	/* Finally, create the instance. */
