@@ -31,6 +31,10 @@
 #define SYNCPTR_FMTPRE  SYNCPTR_PREF "(%s) for '%s A/AAAA %s' "
 #define SYNCPTR_FMTPOST ldap_modop_str(mod_op), a_name_str, ip_str
 
+#if LIBDNS_VERSION_MAJOR < 1600
+#define dns_name_copynf(src, dst) dns_name_copy((src), (dst), NULL)
+#endif
+
 /*
  * Event for asynchronous PTR record synchronization.
  */
@@ -134,7 +138,7 @@ sync_ptr_find(dns_zt_t *zonetable, zone_register_t *zone_register, const int af,
 	 * @example
 	 * 192.168.0.1 -> 1.0.168.192.in-addr.arpa
 	 */
-	CHECK(dns_byaddr_createptrname2(&isc_ip, 0, ptr_name));
+	CHECK(dns_byaddr_createptrname(&isc_ip, 0, ptr_name));
 
 	/* Find an active zone containing owner name of the PTR record. */
 	result = dns_zt_find(zonetable, ptr_name, 0, NULL, zone);
@@ -392,14 +396,12 @@ sync_ptr_init(isc_mem_t *mctx, dns_zt_t * zonetable,
 						LDAPDB_EVENT_SYNCPTR,
 						sync_ptr_handler, NULL,
 						sizeof(sync_ptrev_t));
-	if (ev == NULL)
-		CLEANUP_WITH(ISC_R_NOMEMORY);
 
 	ev->mctx = NULL;
 	isc_mem_attach(mctx, &ev->mctx);
 	INIT_BUFFERED_NAME(ev->a_name);
 	INIT_BUFFERED_NAME(ev->ptr_name);
-	CHECK(dns_name_copy(a_name, &ev->a_name, NULL));
+	dns_name_copynf(a_name, &ev->a_name);
 	ev->mod_op = mod_op;
 	strncpy(ev->ip_str, ip_str, sizeof(ev->ip_str));
 	ev->ip_str[sizeof(ev->ip_str) - 1] = '\0';
