@@ -11,6 +11,7 @@
 #include <isc/time.h>
 #include <isc/util.h>
 
+#include "config.h"
 #include "ldap_helper.h"
 #include "util.h"
 #include "semaphore.h"
@@ -157,7 +158,7 @@ finish(isc_task_t *task, isc_event_t *event) {
 	return;
 }
 
-static isc_result_t ATTR_NONNULLS ATTR_CHECKRESULT
+static void ATTR_NONNULLS
 sync_finishev_create(sync_ctx_t *sctx, ldap_instance_t *inst,
 		      sync_barrierev_t **evp) {
 	sync_barrierev_t *ev = NULL;
@@ -170,14 +171,12 @@ sync_finishev_create(sync_ctx_t *sctx, ldap_instance_t *inst,
 				sctx, LDAPDB_EVENT_SYNCREPL_BARRIER,
 				finish, NULL,
 				sizeof(sync_barrierev_t));
-	if (ev == NULL)
-		return ISC_R_NOMEMORY;
 
 	ev->inst = inst;
 	ev->sctx = sctx;
 	*evp = ev;
 
-	return ISC_R_SUCCESS;
+	return;
 }
 
 /**
@@ -198,37 +197,35 @@ sync_finishev_create(sync_ctx_t *sctx, ldap_instance_t *inst,
  */
 void
 barrier_decrement(isc_task_t *task, isc_event_t *event) {
-	isc_result_t result = ISC_R_SUCCESS;
 	sync_barrierev_t *bev = NULL;
-	sync_barrierev_t *fev = NULL;
-	isc_event_t *ev = NULL;
 	uint32_t cnt;
-	bool locked = false;
 
 	REQUIRE(ISCAPI_TASK_VALID(task));
 	REQUIRE(event != NULL);
 
 	bev = (sync_barrierev_t *)event;
+#if LIBDNS_VERSION_MAJOR < 1600
 	isc_refcount_decrement(&bev->sctx->task_cnt, &cnt);
-	if (cnt == 0) {
+#else
+	cnt = isc_refcount_decrement(&bev->sctx->task_cnt);
+#endif
+	if (cnt == 1) {
+		sync_barrierev_t *fev = NULL;
+		isc_event_t *ev = NULL;
+
 		log_debug(1, "sync_barrier_wait(): barrier reached");
 		LOCK(&bev->sctx->mutex);
-		locked = true;
-		CHECK(sync_finishev_create(bev->sctx, bev->inst, &fev));
+		sync_finishev_create(bev->sctx, bev->inst, &fev);
 		ev = (isc_event_t *)fev;
 		isc_task_send(ldap_instance_gettask(bev->sctx->inst), &ev);
+		UNLOCK(&bev->sctx->mutex);
 	}
 
-cleanup:
-	if (locked)
-		UNLOCK(&bev->sctx->mutex);
-	if (result != ISC_R_SUCCESS)
-		log_error_r("barrier_decrement() failed");
 	isc_event_free(&event);
 	return;
 }
 
-static isc_result_t ATTR_NONNULLS ATTR_CHECKRESULT
+static void ATTR_NONNULLS
 sync_barrierev_create(sync_ctx_t *sctx, ldap_instance_t *inst,
 		      sync_barrierev_t **evp) {
 	sync_barrierev_t *ev = NULL;
@@ -241,14 +238,12 @@ sync_barrierev_create(sync_ctx_t *sctx, ldap_instance_t *inst,
 				sctx, LDAPDB_EVENT_SYNCREPL_BARRIER,
 				barrier_decrement, NULL,
 				sizeof(sync_barrierev_t));
-	if (ev == NULL)
-		return ISC_R_NOMEMORY;
 
 	ev->inst = inst;
 	ev->sctx = sctx;
 	*evp = ev;
 
-	return ISC_R_SUCCESS;
+	return;
 }
 
 /**
@@ -271,19 +266,21 @@ sync_ctx_init(isc_mem_t *mctx, ldap_instance_t *inst, sync_ctx_t **sctxp) {
 
 	REQUIRE(sctxp != NULL && *sctxp == NULL);
 
-	CHECKED_MEM_GET_PTR(mctx, sctx);
+	sctx = isc_mem_get(mctx, sizeof(*(sctx)));
 	ZERO_PTR(sctx);
 	isc_mem_attach(mctx, &sctx->mctx);
 
 	sctx->inst = inst;
 
-	CHECK(isc_mutex_init(&sctx->mutex));
+	/* isc_mutex_init failures are now fatal */
+	isc_mutex_init(&sctx->mutex);
 	lock_ready = true;
-	CHECK(isc_condition_init(&sctx->cond));
+	/* isc_mutex_init failures are now fatal */
+	isc_condition_init(&sctx->cond);
 	cond_ready = true;
 
 	/* refcount includes ldap_inst->task implicitly */
-	CHECK(isc_refcount_init(&sctx->task_cnt, 0));
+	isc_refcount_init(&sctx->task_cnt, 0);
 	refcount_ready = true;
 
 	ISC_LIST_INIT(sctx->tasks);
@@ -297,8 +294,10 @@ sync_ctx_init(isc_mem_t *mctx, ldap_instance_t *inst, sync_ctx_t **sctxp) {
 	return ISC_R_SUCCESS;
 
 cleanup:
-	if (lock_ready == true)
-		DESTROYLOCK(&sctx->mutex);
+	if (lock_ready == true) {
+		/* isc_mutex_destroy failures are now fatal */
+		isc_mutex_destroy(&sctx->mutex);
+	}
 	if (cond_ready == true)
 		RUNTIME_CHECK(isc_condition_destroy(&sctx->cond)
 			      == ISC_R_SUCCESS);
@@ -330,14 +329,19 @@ sync_ctx_free(sync_ctx_t **sctxp) {
 		next_taskel = NEXT(taskel, link);
 		UNLINK(sctx->tasks, taskel, link);
 		isc_task_detach(&taskel->task);
+#if LIBDNS_VERSION_MAJOR < 1600
 		isc_refcount_decrement(&sctx->task_cnt, NULL);
+#else
+		(void)isc_refcount_decrement(&sctx->task_cnt);
+#endif
 		SAFE_MEM_PUT_PTR(sctx->mctx, taskel);
 	}
 	RUNTIME_CHECK(isc_condition_destroy(&sctx->cond) == ISC_R_SUCCESS);
 	isc_refcount_destroy(&sctx->task_cnt);
 	UNLOCK(&sctx->mutex);
 
-	DESTROYLOCK(&(*sctxp)->mutex);
+	/* isc_mutex_destroy is void now */
+	isc_mutex_destroy(&(*sctxp)->mutex);
 	MEM_PUT_AND_DETACH(*sctxp);
 }
 
@@ -444,14 +448,13 @@ sync_state_reset(sync_ctx_t *sctx) {
  */
 isc_result_t
 sync_task_add(sync_ctx_t *sctx, isc_task_t *task) {
-	isc_result_t result = ISC_R_SUCCESS;
-	uint32_t cnt;
 	task_element_t *newel = NULL;
+	uint32_t cnt;
 
 	REQUIRE(sctx != NULL);
 	REQUIRE(ISCAPI_TASK_VALID(task));
 
-	CHECKED_MEM_GET_PTR(sctx->mctx, newel);
+	newel = isc_mem_get(sctx->mctx, sizeof(*(newel)));
 	ZERO_PTR(newel);
 	ISC_LINK_INIT(newel, link);
 	newel->task = NULL;
@@ -460,14 +463,17 @@ sync_task_add(sync_ctx_t *sctx, isc_task_t *task) {
 	LOCK(&sctx->mutex);
 	REQUIRE(sctx->state == sync_configinit || sctx->state == sync_datainit);
 	ISC_LIST_APPEND(sctx->tasks, newel, link);
+#if LIBDNS_VERSION_MAJOR < 1600
 	isc_refcount_increment0(&sctx->task_cnt, &cnt);
+#else
+	cnt = isc_refcount_increment0(&sctx->task_cnt);
+#endif
 	UNLOCK(&sctx->mutex);
 
 	log_debug(2, "adding task %p to syncrepl list; %u tasks in list",
 		  task, cnt);
 
-cleanup:
-	return result;
+	return ISC_R_SUCCESS;
 }
 
 /**
@@ -483,7 +489,6 @@ cleanup:
  */
 isc_result_t
 sync_barrier_wait(sync_ctx_t *sctx, ldap_instance_t *inst) {
-	isc_result_t result;
 	isc_event_t *ev = NULL;
 	sync_barrierev_t *bev = NULL;
 	sync_state_t barrier_state;
@@ -518,7 +523,7 @@ sync_barrier_wait(sync_ctx_t *sctx, ldap_instance_t *inst) {
 	     taskel != NULL;
 	     taskel = next_taskel) {
 		bev = NULL;
-		CHECK(sync_barrierev_create(sctx, inst, &bev));
+		sync_barrierev_create(sctx, inst, &bev);
 		next_taskel = NEXT(taskel, link);
 		UNLINK(sctx->tasks, taskel, link);
 		ev = (isc_event_t *)bev;
@@ -531,12 +536,12 @@ sync_barrier_wait(sync_ctx_t *sctx, ldap_instance_t *inst) {
 		WAIT(&sctx->cond, &sctx->mutex);
 	log_debug(1, "sync_barrier_wait(): all events were processed");
 
-cleanup:
 	UNLOCK(&sctx->mutex);
 
-	if (ev != NULL)
+	if (ev != NULL) {
 		isc_event_free(&ev);
-	return result;
+	}
+	return ISC_R_SUCCESS;
 }
 
 /**
@@ -594,12 +599,10 @@ sync_event_send(sync_ctx_t *sctx, isc_task_t *task, ldap_syncreplevent_t **ev,
 	isc_result_t result;
 	isc_time_t abs_timeout;
 	uint32_t seqid;
-	bool locked = false;
 
 	REQUIRE(sctx != NULL);
 
 	LOCK(&sctx->mutex);
-	locked = true;
 	/* overflow is not a problem as long as the modulo is smaller than
 	 * constant used by sync_concurr_limit_wait() */
 	(*ev)->seqid = seqid = ++sctx->next_id % 0xffffffff;
@@ -617,8 +620,7 @@ sync_event_send(sync_ctx_t *sctx, isc_task_t *task, ldap_syncreplevent_t **ev,
 	result = ISC_R_SUCCESS;
 
 cleanup:
-	if (locked == true)
-		UNLOCK(&sctx->mutex);
+	UNLOCK(&sctx->mutex);
 	return result;
 }
 
